@@ -766,19 +766,21 @@ class LasV_14TestCase(unittest.TestCase):
         self.assertTrue(all(scanner_channel == File2.get_scanner_channel()))
         File2.close(ignore_header_changes = True)
 
+
     def test_vlr_defined_dimensions(self):
-        """Testing v1.4 VLR defined dimensions (LL API)"""
+        """Testingi multiple v1.4 VLR defined dimensions (LL API)"""
         new_header = self.File1.header.copy()
         # Test basic numeric dimension
-        new_dim_record1 = header.ExtraBytesStruct(name = "Test Dimension 1234", data_type = 5)
+        new_dim_record1 = header.ExtraBytesStruct(name = "test dimension 1234", data_type = 5)
         # Test string dimension (len 3)
-        new_dim_record2 = header.ExtraBytesStruct(name = "Test Dimension 5678", data_type = 22)
+        new_dim_record2 = header.ExtraBytesStruct(name = "test dimension 5678", data_type = 22)
         # Test integer array dimension (len 3)
-        new_dim_record3 = header.ExtraBytesStruct(name = "Test Dimension 9", data_type =  26)
+        new_dim_record3 = header.ExtraBytesStruct(name = "test dimension 9", data_type =  26)
         new_VLR_rec = header.VLR(user_id = "LASF_Spec", record_id = 4,
-                VLR_body = (new_dim_record1.to_byte_string() + new_dim_record2.to_byte_string() + new_dim_record3.to_byte_string()))
+            VLR_body = (new_dim_record1.to_byte_string() + new_dim_record2.to_byte_string() + new_dim_record3.to_byte_string()))
         new_header.data_record_length += (19)
-        File2 = File.File(self.output_tempfile, mode = "w", header = new_header, vlrs = [new_VLR_rec], evlrs = self.File1.header.evlrs)
+        File2 = File.File(self.output_tempfile, mode = "w", header = new_header, 
+                vlrs = [new_VLR_rec], evlrs = self.File1.header.evlrs)
 
         File2.X = self.File1.X
 
@@ -791,6 +793,62 @@ class LasV_14TestCase(unittest.TestCase):
         File2.close(ignore_header_changes = True)
 
 
+    def test_vlr_defined_dimension_dtypes(self):
+        """Testing v1.4 VLR defined dimension data types (LL API)"""
+        from laspy.util import edim_fmt_dict, fmtLen, LEfmt,defaults,defaults_test
+
+        for dfList in [defaults, defaults_test]:
+
+            for i in range(1,31):
+                print("...data format: "+ str(i))
+                # Create a new header
+                new_header = self.File1.header.copy()
+                # Create new dimension
+                dimname = "test_dimension_" + str(i)
+                new_dimension = header.ExtraBytesStruct(
+                    name = dimname, data_type = i) 
+                # Collect bytes for new dimension specification 
+                new_dim_raw = new_dimension.to_byte_string() 
+                # Create a VLR defining our new dim
+                new_VLR_rec = header.VLR(user_id = "LASF_Spec", record_id = 4,
+                    description = "Testing Extra Bytes.",
+                    VLR_body = (new_dim_raw))
+
+                # Figure out how much we need to pad the point records
+                new_dim_fmt = edim_fmt_dict[i]
+                new_dim_num = new_dim_fmt[1]
+                new_dim_bytelen = fmtLen[LEfmt[new_dim_fmt[0]]]
+                new_total_bytes = new_dim_bytelen*new_dim_num 
+
+                new_header.data_record_length += (new_total_bytes)
+
+                File2 = File.File(self.output_tempfile, mode = "w", header = new_header, 
+                    vlrs = [new_VLR_rec], evlrs = self.File1.header.evlrs)
+    
+                File2.X = self.File1.X
+    
+                dim_default = dfList[LEfmt[edim_fmt_dict[i][0]]]
+                is_str = type(dim_default) == type(" ")
+                if new_dim_num == 1:
+                    new_dim_val = [dim_default]*len(self.File1)
+                elif is_str:
+                    new_dim_val = [dim_default*new_dim_num]*len(self.File1)
+                else: 
+                    new_dim_val = [[dim_default]*new_dim_num]*len(self.File1)
+
+                File2._writer.set_dimension(dimname, new_dim_val)
+                current_dim_val = File2._writer.get_dimension(dimname)
+
+                if (is_str):
+                    self.assertEqual([x.encode() for x in new_dim_val],
+                            current_dim_val.tolist(), msg = "Problem with data format " + str(i))
+                else:
+                    self.assertEqual(current_dim_val.tolist(), new_dim_val,
+                        msg = "Problem with data format " + str(i))
+                
+                File2.close(ignore_header_changes = True)
+    
+    
     def test_vlr_defined_dimensions2(self):
         """Testing VLR defined dimensions (HL API)"""
         File2 = File.File(self.output_tempfile, mode = "w", header = self.File1.header)
@@ -802,6 +860,17 @@ class LasV_14TestCase(unittest.TestCase):
         self.assertEqual(File2.test_dimension2[123], 0)
         File2.close(ignore_header_changes = True)
 
+    def test_vlr_defined_dimensions3(self):
+        """Testing VLR defined dimensions (HL API)"""
+        File2 = File.File(self.output_tempfile, mode = "w", header = self.File1.header)
+        File2.define_new_dimension("Test_Dimension", 5, "This is a test.")
+
+        File2.X = self.File1.X
+        self.assertEqual(File2.Test_Dimension[500], 0)
+        File2.close(ignore_header_changes = True)
+
+
+
     def tearDown(self):
         self.File1.close()
         really_remove(self.tempfile)
@@ -837,6 +906,29 @@ class LazNoPointMapTestCase(unittest.TestCase):
         with self.assertRaises(Exception):
             plas = File.File(simple, mode="rr")
 
+class LazReaderTestCase(unittest.TestCase):
+    simple_z = os.path.join(os.path.dirname(__file__), 'data', 'simple.laz')
+    simple_s = os.path.join(os.path.dirname(__file__), 'data', 'simple.las')
+
+    def test_open(self):
+        """Test opening LAZ file with lazperf"""
+        plas = File.File(self.simple_z, mode="r")
+        self.assertTrue(plas.header.records_count == 1065)
+        plas.close()
+
+    def test_x(self):
+        """Fetch and test X, Y, Z dimensions"""
+        compressed = File.File(self.simple_z, mode="r")
+        uncompressed = File.File(self.simple_s, mode="r")
+        X_z = list(compressed.X)
+        Y_z = list(compressed.Y)
+        Z_z = list(compressed.Z)
+        X_s = list(uncompressed.X)
+        Y_s = list(uncompressed.Y)
+        Z_s = list(uncompressed.Z)
+        self.assertTrue((list(X_z) == list(X_s)))
+        self.assertTrue((list(Y_z) == list(Y_s)))
+        self.assertTrue((list(Z_z) == list(Z_s)))
 
 def test_laspy():
     reader = unittest.TestLoader().loadTestsFromTestCase(LasReaderTestCase)
@@ -845,11 +937,11 @@ def test_laspy():
     write_mode = unittest.TestLoader().loadTestsFromTestCase(LasWriteModeTestCase)
     las13 = unittest.TestLoader().loadTestsFromTestCase(LasV_13TestCase)
     las14 = unittest.TestLoader().loadTestsFromTestCase(LasV_14TestCase)
-    # laz = unittest.TestLoader().loadTestsFromTestCase(LasLazReaderTestCase)
+    laz = unittest.TestLoader().loadTestsFromTestCase(LazReaderTestCase)
     open_modes = unittest.TestLoader().loadTestsFromTestCase(LazNoPointMapTestCase)
 
     return unittest.TestSuite([reader, writer, header_writer, write_mode,
-                               las13, las14, open_modes])
+                               las13, las14, open_modes, laz])
 
 
 def really_copyfile(src, dst, max_=1):
